@@ -1,7 +1,7 @@
 import tensorflow as tf
 import tensorflow_datasets as tfds
 from tensorflow import keras
-from keras_cv_attention_models.imagenet.data import init_mean_std_by_rescale_mode, random_crop_fraction
+from keras_cv_attention_models.imagenet.data import init_mean_std_by_rescale_mode, tf_imread, random_crop_fraction
 from keras_cv_attention_models.coco import anchors_func
 
 COCO_LABELS = """person, bicycle, car, motorcycle, airplane, bus, train, truck, boat, traffic light, fire hydrant, stop sign,
@@ -11,7 +11,7 @@ COCO_LABELS = """person, bicycle, car, motorcycle, airplane, bus, train, truck,
     cake, chair, couch, potted plant, bed, dining table, toilet, tv, laptop, mouse, remote, keyboard, cell phone, microwave, oven,
     toaster, sink, refrigerator, book, clock, vase, scissors, teddy bear, hair drier, toothbrush"""
 COCO_80_LABEL_DICT = {id: ii.strip() for id, ii in enumerate(COCO_LABELS.split(","))}
-INVALID_ID_90 = [12, 26, 29, 30, 45, 66, 68, 69, 71, 83]
+INVALID_ID_90 = [11, 25, 28, 29, 44, 65, 67, 68, 70, 82]
 COCO_90_LABEL_DICT = {id: ii for id, ii in zip(set(range(90)) - set(INVALID_ID_90), COCO_80_LABEL_DICT.values())}
 COCO_90_LABEL_DICT.update({ii: "Unknown" for ii in INVALID_ID_90})
 COCO_80_to_90_LABEL_DICT = {id_80: id_90 for id_80, id_90 in enumerate(set(range(90)) - set(INVALID_ID_90))}
@@ -151,6 +151,25 @@ def random_flip_left_right_with_bboxes(image, bboxes, probability=0.5):
     )
 
 
+def random_hsv(image, hue_delta=0.015, saturation_delta=0.7, brightness_delta=0.4, show_sample=False):
+    # augment_hsv https://github.com/WongKinYiu/yolor/blob/main/utils/datasets.py#L941
+    if show_sample:
+        import matplotlib.pyplot as plt
+
+        fig = plt.figure()
+        aa = tf.concat([tf.image.adjust_hue(image, -hue_delta), tf.image.adjust_hue(image, hue_delta)], axis=1)
+        bb = tf.concat([tf.image.adjust_saturation(image, 1 - saturation_delta), tf.image.adjust_saturation(image, 1 + saturation_delta)], axis=1)
+        cc = tf.concat([tf.image.adjust_brightness(image, -brightness_delta), tf.image.adjust_brightness(image, brightness_delta)], axis=1)
+        plt.imshow(tf.concat([aa, bb, cc], axis=0))
+        plt.axis("off")
+        plt.tight_layout()
+
+    image = tf.image.random_brightness(image, brightness_delta)
+    image = tf.image.random_saturation(image, 1 - saturation_delta, 1 + saturation_delta)
+    image = tf.image.random_hue(image, hue_delta)
+    return image
+
+
 """ Mosaic mix """
 
 
@@ -209,6 +228,7 @@ class RandomProcessImageWithBboxes:
         random_crop_mode=0,  # 0 for eval mode, (0, 1) for random crop, 1 for random largest crop, > 1 for random scale
         resize_method="bilinear",
         resize_antialias=False,
+        use_hsv_augment=True,
         magnitude=0,
         num_layers=2,
         use_color_increasing=True,
@@ -217,9 +237,12 @@ class RandomProcessImageWithBboxes:
         self.max_labels_per_image = max_labels_per_image
         self.target_shape = target_shape if len(target_shape) == 2 else target_shape[:2]
         self.resize_method, self.resize_antialias, self.random_crop_mode, self.magnitude = resize_method, resize_antialias, random_crop_mode, magnitude
-        if magnitude > 0:
+        if magnitude > 0 and use_hsv_augment:
+            self.randaug_wo_pos = random_hsv
+        elif magnitude > 0:
             from keras_cv_attention_models.imagenet import augment
 
+            # TODO: Need to pick color related methods, "Invert" / "Posterize" may not working well here.
             print(">>>> RandAugment: magnitude = %d" % magnitude)
             self.randaug_wo_pos = augment.RandAugment(
                 num_layers=num_layers,
@@ -229,6 +252,7 @@ class RandomProcessImageWithBboxes:
                 use_positional_related_ops=False,  # Set False to exlude [shear, rotate, translate]
                 **randaug_kwargs,
             )
+            # self.randaug_wo_pos = random_hsv
             # RandAugment positional related ops. Including [shear, rotate, translate], Also returns affine transform matrix
             # self.pos_randaug = augment.PositionalRandAugment(num_layers=num_layers, magnitude=magnitude, **randaug_kwargs)
 
@@ -236,6 +260,8 @@ class RandomProcessImageWithBboxes:
         image = datapoint["image"]
         objects = datapoint["objects"]
         bbox, label, is_crowd = tf.cast(objects["bbox"], tf.float32), objects["label"], objects.get("is_crowd", None)
+        if len(image.shape) < 3:
+            image = tf_imread(image)
         if is_crowd is not None:
             is_not_crowd = is_crowd == False
             bbox, label = tf.boolean_mask(bbox, is_not_crowd), tf.boolean_mask(label, is_not_crowd)
@@ -258,13 +284,13 @@ class RandomProcessImageWithBboxes:
         else:
             image, scale = aspect_aware_resize_and_crop_image(image, self.target_shape, method=self.resize_method, antialias=self.resize_antialias)
             crop_hh, crop_ww = 0, 0
-            # bbox = resize_and_crop_bboxes(bbox, (height, width), self.target_shape, scale=scale)
         bbox = resize_and_crop_bboxes(bbox, (height, width), self.target_shape, scale=scale, offset_y=crop_hh, offset_x=crop_ww)
         bbox, label = refine_bboxes_labels_single(bbox, label)
 
         if self.magnitude > 0:
             image.set_shape([*self.target_shape[:2], 3])
             image = self.randaug_wo_pos(image)
+            # image = random_hsv(image)
             # image, affine_matrix = self.pos_randaug(image)
             # bbox = bboxes_apply_affine(bbox, affine_matrix, input_shape=image.shape)
             # bbox, label = refine_bboxes_labels_single(bbox, label)
@@ -331,6 +357,38 @@ def __bboxes_labels_batch_func__(bboxes, labels, anchors, empty_label, num_class
     return tf.map_fn(bbox_process, bbox_labels)
 
 
+def __yolor_bboxes_labels_batch_func__(bboxes, labels, anchor_ratios, feature_sizes, empty_label, num_classes=80):
+    bbox_labels = tf.concat([bboxes, tf.cast(tf.expand_dims(labels, -1), bboxes.dtype)], axis=-1)
+    bbox_process = lambda xx: tf.cond(
+        tf.reduce_any(xx[:, -1] > 0),  # If contains any valid bbox and label
+        lambda: to_one_hot_with_class_mark(anchors_func.yolor_assign_anchors(xx, anchor_ratios, feature_sizes), num_classes),
+        lambda: empty_label,
+    )
+    return tf.map_fn(bbox_process, bbox_labels)
+
+
+def detection_dataset_from_custom_json(data_path):
+    import json
+
+    with open(data_path, "r") as ff:
+        aa = json.load(ff)
+
+    test_key = "validation" if "validation" in aa else "test"
+    train, test, info = aa["train"], aa[test_key], aa["info"]
+    total_images, num_classes = len(train), info["num_classes"]
+    objects_signature = {"bbox": tf.TensorSpec(shape=(None, 4), dtype=tf.float32), "label": tf.TensorSpec(shape=(None,), dtype=tf.int64)}
+    output_signature = {"image": tf.TensorSpec(shape=(), dtype=tf.string), "objects": objects_signature}
+    train_ds = tf.data.Dataset.from_generator(lambda: (ii for ii in train), output_signature=output_signature)
+    test_ds = tf.data.Dataset.from_generator(lambda: (ii for ii in test), output_signature=output_signature)
+
+    options = tf.data.Options()
+    options.experimental_distribute.auto_shard_policy = tf.data.experimental.AutoShardPolicy.DATA
+    train_ds = train_ds.apply(tf.data.experimental.assert_cardinality(len(train))).with_options(options)
+    test_ds = test_ds.apply(tf.data.experimental.assert_cardinality(len(test))).with_options(options)
+    dataset = {"train": train_ds, test_key: test_ds}
+    return dataset, total_images, num_classes
+
+
 def init_dataset(
     data_name="coco/2017",  # dataset params
     input_shape=(256, 256),
@@ -339,24 +397,30 @@ def init_dataset(
     info_only=False,
     max_labels_per_image=100,
     use_anchor_free_mode=False,
+    use_yolor_anchors_mode=False,
     anchor_pyramid_levels=[3, 7],
-    anchor_aspect_ratios=[1, 2, 0.5],  # [1, 2, 0.5] matches efficientdet anchors format. Force using [1] if use_anchor_free_mode
-    anchor_num_scales=3,  # Force using 1 if use_anchor_free_mode
-    anchor_scale=4,  # Force using 1 if use_anchor_free_mode
-    anchor_grid_zero_start="auto",  # False for anchor_free_mode, True for others.
+    anchor_scale=4,  # For efficientdet anchors only
     rescale_mode="torch",  # rescale mode, ["tf", "torch"], or specific `(mean, std)` like `(128.0, 128.0)`
     random_crop_mode=1.0,  # 0 for eval mode, (0, 1) for random crop, 1 for random largest crop, > 1 for random scale
     mosaic_mix_prob=0.0,
     resize_method="bilinear",  # ["bilinear", "bicubic"]
     resize_antialias=False,
+    use_hsv_augment=True,  # Use hsv augment instead of randaug color related. Generally using in other training frameworks.
     magnitude=0,
     num_layers=2,
+    seed=None,
     **augment_kwargs,  # Too many...
 ):
-    try_gcs = True if len(tf.config.list_logical_devices("TPU")) > 0 else False
-    dataset, info = tfds.load(data_name, with_info=True, try_gcs=try_gcs)
-    num_classes = info.features["objects"]["label"].num_classes
-    total_images = info.splits["train"].num_examples
+    # anchor_aspect_ratios=[1, 2, 0.5],  # [1, 2, 0.5] matches efficientdet anchors format. Force using [1] if use_anchor_free_mode
+    # anchor_num_scales=3,  # Force using 1 if use_anchor_free_mode
+    # anchor_grid_zero_start="auto",  # False for anchor_free_mode, True for others.
+    if data_name.endswith(".json"):
+        dataset, total_images, num_classes = detection_dataset_from_custom_json(data_name)
+    else:
+        try_gcs = True if len(tf.config.list_logical_devices("TPU")) > 0 else False
+        dataset, info = tfds.load(data_name, with_info=True, try_gcs=try_gcs)
+        num_classes = info.features["objects"]["label"].num_classes
+        total_images = info.splits["train"].num_examples
     steps_per_epoch = int(tf.math.ceil(total_images / float(batch_size)))
     if info_only:
         return total_images, num_classes, steps_per_epoch
@@ -369,11 +433,12 @@ def init_dataset(
         random_crop_mode=random_crop_mode,
         resize_method=resize_method,
         resize_antialias=resize_antialias,
+        use_hsv_augment=use_hsv_augment,
         magnitude=magnitude,
         num_layers=num_layers,
         **augment_kwargs,
     )
-    train_dataset = dataset["train"].shuffle(buffer_size).map(train_process).batch(batch_size)
+    train_dataset = dataset["train"].shuffle(buffer_size, seed=seed).map(train_process).batch(batch_size)
     # return train_dataset
 
     if mosaic_mix_prob > 0:
@@ -391,17 +456,24 @@ def init_dataset(
         pos_aug = PositionalRandAugmentWithBboxes(magnitude, num_layers, max_labels_per_image, **augment_kwargs)
         train_dataset = train_dataset.map(pos_aug, num_parallel_calls=AUTOTUNE)
 
-    mean, std = init_mean_std_by_rescale_mode(rescale_mode)
-    rescaling = lambda xx: (xx - mean) / std
     if use_anchor_free_mode:
+        # Don't need anchors here, anchor assigning is after getting model predictions.
         bbox_process = lambda bb: to_one_hot_with_class_mark(tf.concat([bb[0], tf.cast(tf.expand_dims(bb[1], -1), bb[0].dtype)], axis=-1), num_classes)
+    elif use_yolor_anchors_mode:
+        anchor_ratios, feature_sizes = anchors_func.get_yolor_anchors(input_shape[:2], anchor_pyramid_levels, is_for_training=True)
+        total_anchors = tf.cast(anchor_ratios.shape[1] * tf.reduce_sum(feature_sizes[:, 0] * feature_sizes[:, 1]), tf.int32)
+        empty_label = tf.zeros([total_anchors, 4 + num_classes + 1])  # All 0
+        bbox_process = lambda bb: __yolor_bboxes_labels_batch_func__(bb[0], bb[1], anchor_ratios, feature_sizes, empty_label, num_classes)
     else:
-        grid_zero_start = True if anchor_grid_zero_start == "auto" else anchor_grid_zero_start
-        anchors = anchors_func.get_anchors(input_shape[:2], anchor_pyramid_levels, anchor_aspect_ratios, anchor_num_scales, anchor_scale, grid_zero_start)
+        # grid_zero_start = True if anchor_grid_zero_start == "auto" else anchor_grid_zero_start
+        aspect_ratios, num_scales, anchor_scale, grid_zero_start = [1, 2, 0.5], 3, 4, False  # Use this till meet some others new
+        anchors = anchors_func.get_anchors(input_shape[:2], anchor_pyramid_levels, aspect_ratios, num_scales, anchor_scale, grid_zero_start)
         num_anchors = anchors.shape[0]
         empty_label = tf.zeros([num_anchors, 4 + num_classes + 1])  # All 0
         bbox_process = lambda bb: __bboxes_labels_batch_func__(bb[0], bb[1], anchors, empty_label, num_classes)
 
+    mean, std = init_mean_std_by_rescale_mode(rescale_mode)
+    rescaling = lambda xx: (xx - mean) / std
     train_dataset = train_dataset.map(lambda xx, yy: (rescaling(xx), bbox_process(yy)), num_parallel_calls=AUTOTUNE)
     train_dataset = train_dataset.prefetch(buffer_size=AUTOTUNE)
     # return train_dataset
@@ -412,7 +484,6 @@ def init_dataset(
         test_process = RandomProcessImageWithBboxes(target_shape=input_shape, resize_method=resize_method, resize_antialias=resize_antialias, magnitude=-1)
         test_dataset = test_dataset.map(test_process).batch(batch_size).map(lambda xx, yy: (rescaling(xx), bbox_process(yy)))
 
-    # Also returns anchors, it's needed for anchor_freee mode
     return train_dataset, test_dataset, total_images, num_classes, steps_per_epoch
 
 
@@ -450,7 +521,11 @@ def show_image_with_bboxes(image, bboxes, labels=None, confidences=None, is_bbox
 
         if labels is not None:
             label = int(labels[id])
-            label = COCO_90_LABEL_DICT[label] if num_classes == 90 else COCO_80_LABEL_DICT[label]
+            if num_classes == 90:
+                label = COCO_90_LABEL_DICT[label]
+            elif num_classes == 80:
+                label = COCO_80_LABEL_DICT[label]
+
             if confidences is not None:
                 label += ": {:.4f}".format(float(confidences[id]))
             color = ax.lines[-1].get_color()
@@ -462,7 +537,9 @@ def show_image_with_bboxes(image, bboxes, labels=None, confidences=None, is_bbox
     return ax
 
 
-def show_batch_sample(dataset, rescale_mode="torch", rows=-1, label_font_size=8, base_size=3, use_anchor_free_mode=False, **anchor_kwargs):
+def show_batch_sample(
+    dataset, rescale_mode="torch", rows=-1, label_font_size=8, base_size=3, use_anchor_free_mode=False, use_yolor_anchors_mode=False, **anchor_kwargs
+):
     import matplotlib.pyplot as plt
     from keras_cv_attention_models.visualizing import get_plot_cols_rows
 
@@ -472,8 +549,12 @@ def show_batch_sample(dataset, rescale_mode="torch", rows=-1, label_font_size=8,
         images, labels = dataset.as_numpy_iterator().next()
     mean, std = init_mean_std_by_rescale_mode(rescale_mode)
     images = (images * std + mean) / 255
-    if not use_anchor_free_mode:
-        anchors = anchors_func.get_anchors(images.shape[1:-1], **anchor_kwargs)
+    if use_yolor_anchors_mode:
+        pyramid_levels = anchors_func.get_pyramid_levels_by_anchors(images.shape[1:-1], labels.shape[1])
+        anchors = anchors_func.get_yolor_anchors(images.shape[1:-1], pyramid_levels=pyramid_levels, is_for_training=False)
+    elif not use_anchor_free_mode:
+        pyramid_levels = anchors_func.get_pyramid_levels_by_anchors(images.shape[1:-1], labels.shape[1])
+        anchors = anchors_func.get_anchors(images.shape[1:-1], pyramid_levels, **anchor_kwargs)
 
     cols, rows = get_plot_cols_rows(len(images), rows, ceil_mode=True)
     fig, axes = plt.subplots(rows, cols, figsize=(base_size * cols, base_size * rows))
@@ -489,7 +570,13 @@ def show_batch_sample(dataset, rescale_mode="torch", rows=-1, label_font_size=8,
             valid_label = tf.cast(tf.argmax(valid_preds[:, 4:-1], axis=-1), valid_preds.dtype)
             valid_preds = tf.concat([valid_preds[:, :4], tf.expand_dims(valid_label, -1)], axis=-1)
 
-        if not use_anchor_free_mode:
+        if use_yolor_anchors_mode:
+            valid_anchors = anchors[pick]
+            decoded_centers = (valid_preds[:, :2] + 0.5) * valid_anchors[:, 4:] + valid_anchors[:, :2]
+            decoded_hw = valid_preds[:, 2:4] * valid_anchors[:, 4:]
+            decoded_corner = anchors_func.center_yxhw_to_corners_nd(tf.concat([decoded_centers, decoded_hw], axis=-1))
+            valid_preds = tf.concat([decoded_corner, valid_preds[:, -1:]], axis=-1)
+        elif not use_anchor_free_mode:
             valid_anchors = anchors[pick]
             valid_preds = anchors_func.decode_bboxes(valid_preds, valid_anchors)
         show_image_with_bboxes(image, valid_preds[:, :4], valid_preds[:, -1], ax=ax, label_font_size=label_font_size)
